@@ -331,7 +331,7 @@ def cosine_pairwise_distance(a, return_matrix=False):
             for j in range(i):
                 out_mat[i][j] = np.dot(a[i], a[j])
         out_mat = out_mat + out_mat.T
-        np.fill_diagonal(out_mat,1) 
+        np.fill_diagonal(out_mat,1)
         return out_mat
     else:
         out = np.zeros((len(perm), 1))
@@ -889,7 +889,7 @@ def vector_to_matrix_distance(u, m, metric, metric_name):
         return cosine_vector_to_matrix(u, m)
 
     n = m.shape[0]
-    out = np.zeros((n))
+    out = np.zeros((n), dtype=u.dtype)
     for i in range(n):
         out[i] = metric(u, m[i])
     return out
@@ -930,7 +930,7 @@ def matrix_to_matrix_distance(a, b, metric, metric_name):
     if metric_name == "cosine":
         return cosine_matrix_to_matrix(a, b)
     n, m = a.shape[0], b.shape[0]
-    out = np.zeros((n, m))
+    out = np.zeros((n, m), dtype=a.dtype)
     for i in range(n):
         for j in range(m):
             out[i][j] = metric(a[i], b[j])
@@ -981,13 +981,13 @@ def matrix_pairwise_distance(a, metric, metric_name, return_matrix=False):
         rows = np.arange(n)
         perm = [(rows[i], rows[j]) for i in range(n) for j in range(i + 1, n)]
         if return_matrix:
-            out_mat = np.zeros((n, n))
+            out_mat = np.zeros((n, n), dtype=a.dtype)
             for i in range(n):
                 for j in range(i):
                     out_mat[i][j] = metric(a[i], a[j])
             return out_mat + out_mat.T
         else:
-            out = np.zeros((len(perm), 1))
+            out = np.zeros((len(perm), 1), dtype=a.dtype)
             for i in range(len(perm)):
                 out[i] = metric(a[perm[i][0]], a[perm[i][1]])
             return out
@@ -1306,13 +1306,15 @@ def confusion_matrix(targets, preds, w=None, normalize=None):
 
 
 @jit(nopython=True, fastmath=True)
-def accuracy_score(targets, preds, w=None, normalize=True):
+def accuracy_score(targets, preds, cm=None, w=None, normalize=True):
     """
     :purpose:
     Calculates the accuracy score between a discrete target and pred array
 
     :params:
     targets, preds : discrete input arrays, both of shape (n,)
+    cm             : if you have previously calculated a confusion matrix, pass it here to save the computation.
+                     set as None, which makes the function calculate the confusion matrix
     w              : weights at each index of true and pred. array of shape (n,)
                      if no w is set, it is initialized as an array of ones
                      such that it will have no impact on the output
@@ -1331,11 +1333,16 @@ def accuracy_score(targets, preds, w=None, normalize=True):
     0.4903
     """
     w = init_w(w, len(targets))
+    if cm is None:
+        cm = confusion_matrix(targets, preds, w=w)
+    n = cm.shape[0]
+
     num, denom = 0, 0
-    for i in range(len(targets)):
-        if targets[i] == preds[i]:
-            num += w[i]
-        denom += w[i]
+    for i in range(n):
+        num += cm[i][i]  # sum of the diagonal = true results
+        for j in range(n):
+            denom += cm[i][j]  # total sum = true and false results
+
     return num / denom if normalize else num
 
 
@@ -1372,15 +1379,119 @@ def balanced_accuracy_score(targets, preds, cm=None, w=None, adjusted=False):
     n = cm.shape[0]
     diag, row_sums = np.zeros(n), np.zeros(n)
     for i in range(n):
-        diag[i] = cm[i][i]
+        diag[i] = cm[i][i]  # sum of the diagonal = true results
         for j in range(n):
-            row_sums[i] += cm[i][j]
+            row_sums[i] += cm[i][j]  # sums of the rows = original ground truth class assignments
 
-    class_div = diag / row_sums
+    class_div = diag / row_sums  # fraction of correctly recovered targets per class
     div_mean = 0
     for i in range(n):
         div_mean += class_div[i]
-    div_mean /= n
+    div_mean /= n  # mean fraction of correctly recovered targets
+
+    if adjusted:
+        div_mean -= 1 / n
+        div_mean /= 1 - 1 / n
+    return div_mean
+
+
+@jit(nopython=True, fastmath=True)
+def mean_predictive_value(targets, preds, cm=None, w=None, adjusted=False):
+    """
+    :purpose:
+    Calculates the mean predictive value between a discrete target and pred array
+
+    :params:
+    targets, preds : discrete input arrays, both of shape (n,)
+    cm             : if you have previously calculated a confusion matrix, pass it here to save the computation.
+                     set as None, which makes the function calculate the confusion matrix
+    w              : weights at each index of true and pred. array of shape (n,)
+                     if no w is set, it is initialized as an array of ones
+                     such that it will have no impact on the output
+    adjusted       : bool. if true, adjust the output for chance (making 0 the worst
+                     and 1 the best score). defaults to false
+
+    :returns:
+    mean_predictive_value : float, the mean predictive value of the targets and preds array
+
+    :example:
+    >>> from fastdist import fastdist
+    >>> import numpy as np
+    >>> true = np.random.RandomState(seed=0).randint(2, size=10000)
+    >>> pred = np.random.RandomState(seed=1).randint(2, size=10000)
+    >>> fastdist.mean_predictive_value(true, pred)
+    0.49030739883826424
+
+    by saskra
+    """
+    w = init_w(w, len(targets))
+    if cm is None:
+        cm = confusion_matrix(targets, preds, w=w)
+    n = cm.shape[0]
+    diag, columns_sums = np.zeros(n), np.zeros(n)
+    for i in range(n):
+        diag[i] = cm[i][i]  # sum of the diagonal = true results
+        for j in range(n):
+            columns_sums[j] += cm[i][j]  # sums of the columns = predictions per class
+
+    class_div = diag / columns_sums  # fraction of true results among the predicted ones per class
+    div_mean = 0
+    for i in range(n):
+        div_mean += class_div[i]
+    div_mean /= n  # mean fraction of true results among the predicted ones
+
+    if adjusted:
+        div_mean -= 1 / n
+        div_mean /= 1 - 1 / n
+    return div_mean
+
+
+@jit(nopython=True, fastmath=True)
+def mean_iou(targets, preds, cm=None, w=None, adjusted=False):
+    """
+    :purpose: Calculates the mean intersection of ground truth and prediction over union
+
+    :params:
+    targets, preds : discrete input arrays, both of shape (n,)
+    cm             : if you have previously calculated a confusion matrix, pass it here to save the computation.
+                     set as None, which makes the function calculate the confusion matrix
+    w              : weights at each index of true and pred. array of shape (n,)
+                     if no w is set, it is initialized as an array of ones
+                     such that it will have no impact on the output
+    adjusted       : bool. if true, adjust the output for chance (making 0 the worst
+                     and 1 the best score). defaults to false
+
+    :returns:
+    mean_iou : float, the mean intersection over union of the targets and preds array
+
+    :example:
+    >>> from fastdist import fastdist
+    >>> import numpy as np
+    >>> true = np.random.RandomState(seed=0).randint(2, size=10000)
+    >>> pred = np.random.RandomState(seed=1).randint(2, size=10000)
+    >>> fastdist.mean_iou(true, pred)
+    0.49030739883826424
+
+    by saskra
+    """
+    w = init_w(w, len(targets))
+    if cm is None:
+        cm = confusion_matrix(targets, preds, w=w)
+    n = cm.shape[0]
+    diag, rows_sums, columns_sums = np.zeros(n), np.zeros(n), np.zeros(n)
+    for i in range(n):
+        for j in range(n):
+            if i == j:
+                diag[i] = cm[i][j]  # sum of the diagonal = true results
+            else:
+                rows_sums[i] += cm[i][j]  # rest of the row = false negative results
+                columns_sums[j] += cm[i][j]  # rest of the column = false positive results
+
+    class_div = diag / (columns_sums + rows_sums + diag)  # intersection over union (Jaccard) per class
+    div_mean = 0
+    for i in range(n):
+        div_mean += class_div[i]
+    div_mean /= n  # mean intersection over union
 
     if adjusted:
         div_mean -= 1 / n
